import json
from pathlib import Path
from typing import Dict, Any, Optional, Union
from importlib import resources

from .schema import AppConfig


def _deep_update(a: Dict[str, Any], b: Dict[str, Any]) -> Dict[str, Any]:
    """Recursively merge b into a (b wins)."""
    for k, v in b.items():
        if isinstance(v, dict) and isinstance(a.get(k), dict):
            a[k] = _deep_update(a[k], v)
        else:
            a[k] = v
    return a


def load_config(
    *,
    client_defaults_path: Optional[Union[str, Path]] = None,
    user_config_path: Optional[Union[str, Path]] = None,
    cli_overrides: Optional[Dict[str, Any]] = None,
) -> AppConfig:
    """Return a fully-merged, validated AppConfig."""
    # 1) library defaults (packaged)
    try:
        base = json.loads(
            resources.files("comfydock_server.config")
            .joinpath("default_config.json")
            .read_text()
        )
    except Exception as e:
        raise RuntimeError(f"Failed to load default config: {e}")

    # 2) optional per-client overrides (ship in the client repo)
    if client_defaults_path and Path(client_defaults_path).exists():
        try:
            with open(client_defaults_path, 'r') as f:
                client_config = json.load(f)
            base = _deep_update(base, client_config)
        except Exception as e:
            raise RuntimeError(f"Failed to load client config from {client_defaults_path}: {e}")

    # 3) optional user-level overrides (e.g. ~/.comfydock/config.json)
    if user_config_path and Path(user_config_path).exists():
        try:
            with open(user_config_path, 'r') as f:
                user_config = json.load(f)
            base = _deep_update(base, user_config)
        except Exception as e:
            raise RuntimeError(f"Failed to load user config from {user_config_path}: {e}")

    # 4) CLI / environment variable overrides (already parsed)
    if cli_overrides:
        base = _deep_update(base, cli_overrides)

    try:
        return AppConfig(**base)
    except Exception as e:
        raise RuntimeError(f"Failed to validate configuration: {e}")